__title__ = "durakonline.py"
__author__ = "Zakovskiy"
__license__ = "MIT"
__copyright__ = "Copyright 2021-2022 Zakovskiy"
__version__ = "3.4.0"

from .durakonline import Client
from .authorization import Authorization
from .game import Game
from .friend import Friend
from .socket_listener import SocketListener

from .utils import objects

from requests import get
from json import loads

__newest__ = loads(get("https://pypi.python.org/pypi/durakonline.py/json").text)["info"]["version"]

if __version__ != __newest__:
    exit(f"New version of {__title__} available: {__newest__} (Using {__version__})")



import base64
import hashlib
from datetime import datetime
from .utils import objects


class Authorization:

    def __init__(self, client) -> None:
        self.client = client

    def get_session_key(self) -> objects.GetSessionKey:
        data = {
            "command": "c",
            "l": "ru",
            "tz": "+02:00",
            "t": datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3]+"Z",
            "pl": self.client.pl,
            "p": 10,
        }
        if self.client.pl == "ios":
            data.update({
                "v": "1.9.1.2",
                "ios": "14.4",
                "d": "iPhone8,4",
                "n": "durak.ios",
            })
        else:
            data.update({
                "v": "1.9.2",
                "d": "xiaomi cactus",
                "and": 28,
                "n": f"durak.{self.client.pl}",
            })
        self.client.send_server(data)
        return objects.GetSessionKey(self.client.listen()).GetSessionKey

    def sign(self, key: str) -> dict:
        hash = base64.b64encode(hashlib.md5(
            (key+"oc3q7ingf978mx457fgk4587fg847").encode()).digest()).decode()
        self.client.send_server(
            {
                "command": "sign",
                "hash": hash,
            }
        )
        return self.client.listen()

    def signin_by_access_token(self, token: str) -> int:
        self.client.token = token
        self.client.send_server(
            {
                "command": "auth",
                "token": self.client.token,
            }
        )
        authorized = self.client._get_data("authorized")
        if authorized["command"] == "err":
            raise objects.Err(authorized)
        self.client.uid = authorized["id"]
        self.client.logger.debug(f"{self.client.tag}: Success auth")
        return authorized["id"]

    def google_auth(self, id_token: str) -> dict:
        self.client.send_server(
            {
                "command": "durak_google_auth",
                "id_token": id_token,
            }
        )
        return self.client.listen()

    def get_captcha(self) -> dict:
        self.client.send_server(
            {
                "command": "get_captcha",
            }
        )
        return self.client._get_data("captcha")

    def register(self, name, captcha: str = '') -> objects.Register:
        self.client.send_server(
            {
                "command": "register",
                "name": name,
                "captcha": captcha,
            }
        )
        return objects.Register(self.client._get_data("set_token")).Register


from .utils import objects


class Friend:

    def __init__(self, client) -> None:
        self.client = client

    def accept(self, friend_id: int) -> None:
        self.client.send_server(
            {
                "command": "friend_accept",
                "id": friend_id
            }
        )

    def delete(self, friend_id: int) -> dict:
        self.client.send_server(
            {
                "command": "friend_delete",
                "id": friend_id
            }
        )
        return self.client._get_data("fl_delete")

    def send_request(self, user_id: int) -> dict:
        self.client.send_server(
            {
                "command": "friend_request",
                "id": user_id
            }
        )
        return self.client.listen()

    def get_list(self) -> [objects.FriendInfo]:
        self.client.send_server(
            {
                "command": "friend_list"
            }
        )
        friends: [objects.FriendInfo] = []
        data = self.client.listen()
        while data["command"] != "img_msg_price":
            if data["command"] == "fl_update":
                friends.append(objects.FriendInfo(data).FriendInfo)
            data = self.client.listen()

        return friends

    def send_message(self, content, to) -> None:
        self.send_server(
            {
                "command": "send_user_msg",
                "msg": content,
                "to": to
            }
        )


class Err(Exception):

	def __init__ (self, data:dir):
		self.json = data
		try: self.code = data["code"]
		except: self.code = None

class GetSessionKey:

	def __init__ (self, data:dir):
		self.json = data
		self.key = None

	@property
	def GetSessionKey (self):
		self.key = self.json["key"]
		return self

class Server:

	def __init__ (self, data:dir):
		self.json = data
		self.time = None
		self.id = None

	@property
	def Server (self):
		self.time = self.json["time"]
		self.id = self.json["id"]
		return self

class SigninByAccessToken:

	def __init__ (self, data:dir):
		self.json = data
		self.id = None

	@property
	def SigninByAccessToken(self):
		self.id = self.json["id"]
		return self

class Register:

	def __init__ (self, data:dir):
		self.json = data
		self.token = None

	@property
	def Register(self):
		self.token = self.json["token"]
		return self

class User:

	def __init__ (self, data: dict):
		self.json = data
		self.id = None
		self.name = None
		self.avatar = None
		self.dtp = None
		self.frame = None
		self.score = None
		self.achieve = None
		self.pw = None

	@property
	def User(self):
		self.id = self.json["id"]
		self.name = self.json["name"]
		self.avatar = self.json["avatar"]
		self.dtp = self.json["dtp"]
		self.frame = self.json["frame"]
		self.score = self.json["score"]
		self.achieve = self.json["achieve"]
		self.pw = self.json["pw"]
		return self
	

class FriendInfo:

	def __init__ (self, data: dict):
		self.json = data
		self.user: User = None
		self.kind = None
		self.new = None

	@property
	def FriendInfo(self):
		self.kind = self.json.get("kind")
		self.new = self.json.get("new")
		self.user = User(self.json.get("user")).User
		return self
	

class UserInfo:

	def __init__ (self, data:dir):
		self.json = data
		self.id = None
		self.name = None
		self.avatar = None
		self.pw = None
		self.ach = []
		self.ach_c = None
		self.t_bronze = None
		self.t_silver = None
		self.t_gold = None
		self.wins = None
		self.wins_season = None
		self.points_win = None
		self.points_win_season = None
		self.score = None
		self.score_season = None
		self.dtp = None
		self.frame = None
		self.assets = []
		self.achieve = None
		self.achieves = []
		self.coll = {}

	@property
	def UserInfo(self):
		self.id = self.json["id"]
		self.name = self.json["name"]
		self.avatar = self.json["avatar"]
		self.pw = self.json["pw"]
		self.ach = self.json["ach"]
		self.ach_c = self.json["achc"]
		self.t_bronze = self.json["t_bronze"]
		self.t_silver = self.json["t_silver"]
		self.t_gold = self.json["t_gold"]
		self.wins = self.json["wins"]
		self.wins_season = self.json["wins_s"]
		self.points_win = self.json["points_win"]
		self.points_win_season = self.json["points_win_s"]
		self.score = self.json["score"]
		self.score_season = self.json["score_s"]
		self.dtp = self.json["dtp"]
		self.frame = self.json["frame"]
		self.assets = self.json["assets"]
		self.achieve = self.json["achieve"]
		self.achieves = self.json["achieves"]
		self.coll = self.json["coll"]
		return self

class Smile:

	def __init__ (self, data:dir):
		self.json = data
		self.id = None
		self.index = None
		self.mask = None
		self.level = None
		self.price = None
		self.name = None
		self.description = None

	@property
	def Smile(self):
		self.id = self.json["id"]
		self.index = self.json["index"]
		self.mask = self.json["mask"]
		self.level = self.json["level"]
		self.price = self.json["price"]
		self.name = self.json["name"]["ru"]
		self.description = self.json["desc"]["ru"]
		return self

class Frame:

	def __init__ (self, data:dir):
		self.json = data

	@property
	def Frame(self):
		return Shirt(self.json).Shirt


class Shirt:

	def __init__ (self, data:dir):
		self.json = data
		self.id = None
		self.index = None
		self.mask = None
		self.level = None
		self.price = None
		self.name = None
		self.description = None
		self.hidden = None
		self.group = None

	@property
	def Shirt(self):
		self.id = self.json["id"]
		self.index = self.json["index"]
		self.mask = self.json["mask"]
		self.level = self.json["level"]
		self.price = self.json["price"]
		self.name = self.json["name"]["ru"]
		try: self.description = self.json["desc"]["ru"]
		except: pass
		self.hidden = self.json["hidden"]
		self.group = self.json["group"]
		return self

class Assets:

	def __init__ (self, data:dir):
		self.json = data
		self.smile = []
		self.shirt = []

	@property
	def Assets (self):
		for smile in self.json["smile"]:
			self.smile.append(Smile(smile).Smile)
		return self

class ItemsPrice:

	def __init__ (self, data:dir):
		self.json = data
		self.ids = []

	@property
	def ItemsPrice(self):
		for id in self.json["ids"]:
			self.ids.append(ItemPrice(id).ItemPrice)
		return self

class PurchaseIds:

	def __init__ (self, data:dir):
		self.json = data
		self.ids = None

	@property
	def PurchaseIds(self):
		self.ids = self.json["ids"]
		return self

class Bets:

	def __init__ (self, data:dir):
		self.json = data
		self.v = []

	@property
	def Bets(self):
		self.v = self.json["v"]
		return self
	

class ItemPrice:

	def __init__(self, data:dir):
		self.json = data
		self.price = None
		self.quantity = None
		self.id = None

	@property
	def ItemPrice(self):
		self.price = self.json["price"]
		self.quantity = self.json["quantity"]
		self.id = self.json["id"]
		return self

class Game:

	def __init__ (self, data:dir):
		self.json = data
		self.id = None
		self.players = None
		self.position = None
		self.deck = None
		self.timeout = None
		self.sw = None
		self.ch = None
		self.dr = None
		self.nb = None
		self.bet = None
		self.fast = None

	@property
	def Game(self):
		self.id = self.json.get("id")
		self.players = self.json.get("players")
		self.position = self.json.get("position")
		self.deck = self.json.get("deck")
		self.timeout = self.json.get("timeout")
		self.sw = self.json.get("sw")
		self.ch = self.json.get("ch")
		self.dr = self.json.get("dr")
		self.nb = self.json.get("nb")
		self.bet = self.json.get("bet")
		self.fast = self.json.get("fast")
		return self


import socket
import json
import requests
import threading
import random


class SocketListener:

    def __init__(self, client):
        self.client = client
        self.alive = False
        self.socket = None
        self.handlers = {}

    def create_connection(self, server_id: str = None, ip: str = None, port: int = None):
        """
        **Parametrs**
            - server_id ::
                "u0" - Diamond
                "u1" - Sapphire
                "u2" - Ruby
                "u3" - Emerald
                "u4" - Amethyst
                "u5" - Aquamarine
                "u6" - Topaz
                "u7" - Opal
                "u8" - Amber
                "u9" - Jade
                "uA" - Onyx
                "uB" - Lazurite
                "uC" - Pearls
                "uD" - Alexandrite
                None - random
        """
        if not ip:
            servers = self.get_servers()["user"]
            server = servers[server_id] if server_id else list(random.choice(list(servers.items())))[1]
            ip = server["host"]
            port = server["port"]
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self.socket.connect((ip, port))
        except Exception as e:
            if "error" in self.handlers:
                self.handlers["error"](e)
        self.alive = True
        threading.Thread(target=self.receive_messages).start()

    def send_server(self, data: dir):
        try:
            self.socket.send((data.pop('command')+json.dumps(data, separators=(',', ':')).replace("{}", '')+'\n').encode())
        except Exception as e:
            if "error" in self.handlers:
                self.handlers["error"](e)

    def get_servers(self):
        try:
            response = requests.get(f"{self.api_url}servers.json").json()
        except Exception as e:
            if "error" in self.handlers:
                self.handlers["error"](e)
        return response

    def event(self, command: str = "all"):
        def register_handler(handler):
            if command in self.handlers:
                self.handlers[command].append(handler)
            else:
                self.handlers[command] = [handler]
            return handler

        return register_handler

    def error(self):
        def register_handler(handler):
            self.handlers["error"] = handler
            return handler

        return register_handler

    def receive_messages(self):
        self.logger.debug(f"{self.tag}: Start listener")
        while self.alive:
            buffer = bytes()
            while self.alive:
                try:
                    r = self.socket.recv(4096)
                except Exception as e:
                    if "error" in self.handlers:
                        self.handlers["error"](e)
                    self.alive = False
                    return
                buffer = buffer + r
                read = len(r)
                if read != -1:
                    if read in [0, 1]: continue
                    try:
                        d = buffer.decode()
                    except:
                        continue
                    if d.endswith('\n'):
                        buffer = bytes()
                        for str in d.strip().split('\n'):
                            str = str[0:-1]
                            pos = str.find('{')
                            command = str[:pos]
                            try:
                                message = json.loads(str[pos:]+"}")
                            except Exception as e:
                                continue
                            message['command'] = command
                            self.logger.debug(f"{self.tag}: {message}")
                            for handler_command in self.handlers:
                                if handler_command in ["all", command]:
                                    for handler in self.handlers[handler_command]:
                                        handler(message)
                            self.receive.append(message)
                    else:
                        continue
                else:
                    self.socket.close()
                    return

    def listen(self, force: bool = False):
        while len(self.receive) == 0:
            if force:
                return {"command": "empty"}
        r = self.receive[0]
        del self.receive[0]
        return r

    def _get_data(self, type, force: bool = False):
        data = self.listen(force=force)
        while 1:
            if data["command"] in [type, "err", "empty", "alert"]:
                return data
            data = self.listen(force=force)


import sys
import socket
import json
import threading
from .utils import objects
from datetime import datetime
from loguru import logger
from .socket_listener import SocketListener

from .authorization import Authorization
from .game import Game
from .friend import Friend


class Client(SocketListener):

    def __init__(self, token: str = None, server_id: str = None, pl: str = "ios",
        debug: bool = False, tag: str = "", ip: str = None,
        port: int = None) -> None:
        super().__init__(self)
        self.api_url = "http://static.rstgames.com/durak/"
        self.pl = pl
        self.tag = tag
        self.uid = None
        self.receive = []
        self.logger = logger
        self.logger.remove()
        self.logger.add(sys.stderr, format="{time:HH:mm:ss.SSS}: {message}", level="DEBUG" if debug else "INFO")
        self.create_connection(server_id, ip, port)
        self.load_classes()
        self.authorization.sign(self.authorization.get_session_key().key)

        if token:
            self.authorization.signin_by_access_token(token)

    def load_classes(self) -> None:
        self.authorization: Authorization = Authorization(self)
        self.game: Game = Game(self)
        self.friend: Friend = Friend(self)

    def get_user_info(self, user_id: int) -> objects.UserInfo:
        self.send_server(
            {
                "command": "get_user_info",
                "id": user_id
            }
        )
        data = self._get_data("user_info")
        if data["command"] != "user_info":
            raise objects.Err(data)
        return objects.UserInfo(data).UserInfo

    def verify_purchase(self, signature, purchase_data) -> None:
        self.send_server(
            {
                "command": "verify_purchase",
                "signature": signature,
                "purchase_data": purchase_data
            }
        )

    def get_purchase_ids(self) -> None:
        self.send_server(
            {
                "command": "get_android_purchase_ids"
            }
        )
        return objects.PurchaseIds(self._get_data("android_purchase_ids")).PurchaseIds

    def get_prem_price(self) -> objects.ItemsPrice:
        self.send_server(
            {
                "command": "get_prem_price"
            }
        )
        return objects.ItemsPrice(self._get_data("prem_price")).ItemsPrice

    def get_points_price(self) -> objects.ItemsPrice:
        self.send_server(
            {
                "command": "get_points_price"
            }
        )
        return objects.ItemsPrice(self._get_data("points_price")).ItemsPrice

    def buy_prem(self, id: int = 0) -> None:
        self.send_server(
            {
                "command": "buy_prem",
                "id": f"com.rstgames.durak.prem.{id}"
            }
        )

    def buy_points(self, id: int = 0) -> dict:
        self.send_server(
            {
                "command": "buy_points",
                "id": f"com.rstgames.durak.points.{id}"
            }
        )

        return self.listen()

    def buy_asset(self, asset_id) -> None:
        self.send_server(
            {
                "command": "buy_asset",
                "id": asset_id
            }
        )

    def get_assets(self) -> objects.Assets:
        self.send_server(
            {
                "command": "get_assets"
            }
        )
        return objects.Assets(self._get_data("assets")).Assets

    def asset_select(self, asset_id) -> None:
        self.send_server(
            {
                "command": "asset_select",
                "id": asset_id
            }
        )

    def achieve_select(self, achieve_id) -> None:
        self.send_server(
            {
                "command": "achieve_select",
                "id": achieve_id
            }
        )

    def complaint(self, to_id) -> None:
        self.send_server(
            {
                "command": "complaint",
                "id": to_id,
            }
        )

    def send_user_message_code(self, code, content) -> None:
        self.send_server(
            {
                "command": "send_user_msg_code",
                "code": code,
                "msg": content
            }
        )

    def delete_message(self, message_id) -> None:
        self.send_server(
            {
                "command": "delete_msg",
                "msg_id": message_id
            }
        )

    def gift_coll_item(self, item_id: id, coll_id: str, to: int) -> dict:
        self.send_server(
            {
                "command": "gift_coll_item",
                "item_id": item_id,
                "coll_id": coll_id,
                "to_id": to
            }
        )
        return self.listen()

    def get_bets(self) -> objects.Bets:
        self.send_server(
            {
                "command": "gb"
            }
        )
        return objects.Bets(self._get_data("bets")).Bets

    def lookup_start(self, betMin: int = 100, pr: bool = False, betMax: int = 2500,
        fast: bool = True, sw: bool = True, nb: list = [False, True], ch: bool = False,
        players: list = [2, 3, 4, 5, 6], deck: list = [24, 36, 52], dr: bool = True) -> None:
        self.send_server(
            {
                "command": "lookup_start",
                "betMin": betMin,
                "pr": [pr, False],
                "betMax": betMax,
                "fast": [fast],
                "sw": [sw],
                "nb": nb,
                "ch": [ch],
                "players": players,
                "deck": deck,
                "dr": [dr],
                "status": "open"
            }
        )

    def lookup_stop(self) -> None:
        self.send_server(
            {
                "command": "lookup_stop"
            }
        )

    def get_server(self) -> None:
        self.send_server(
            {
                "command": "get_server"
            }
        )

    def update_name(self, nickname: str = None) -> None:
        self.send_server(
            {
                "command": "update_name",
                "value": nickname
            }
        )

    def save_note(self, note: str, user_id: int, color: int = 0) -> None:
        self.send_server(
            {
                "command": "save_note",
                "note": note,
                "color": color,
                "id": user_id
            }
        )

    def leaderboard_get_by_user(self, user_id, type: str = "score", season: bool = False) -> dict:
        s = "" if not season else "s_"
        self.send_server(
            {
                "command": s+"lb_get_by_user",
                "user_id": user_id,
                "type": type
            }
        )
        return self._get_data("lb")

    def leaderboard_get_top(self, type: str = "score") -> dict:
        self.send_server(
            {
                "command": "lb_get_top",
                "type": type
            }
        )
        return self._get_data("lb")

    def leaderboard_get_by_place_down(self, place: int = 20, type: str = "score") -> dict:
        self.send_server(
            {
                "command": "lb_get_by_place_down",
                "place": place,
                "type": type
            }
        )
        return self._get_data("lb")


from .utils import objects


class Game:

    def __init__(self, client):
        self.client = client

    def create(self, bet, password: str = "", players: int = 6,
        deck: int = 24, fast: bool = False, sw: bool = True,
        nb: bool = True, ch: bool = False, dr: bool = True) -> objects.Game:
        self.client.send_server(
            {
                "command": "create",
                "bet": bet,
                "password": password,
                "fast": fast,
                "sw": sw,
                "nb": nb,
                "ch": ch,
                "players": players,
                "deck": deck,
                "dr": dr,
            }
        )

        data = self.client._get_data("game")
        if data["command"] == 'err':
            raise objects.Err(data)
        return objects.Game(data).Game

    def join(self, password: str, game_id) -> None:
        self.client.send_server(
            {
                "command": "join",
                "password": password,
                "id": game_id,
            }
        )

    def invite(self, user_id):
        self.client.send_server(
            {
                "command": "invite_to_game",
                "user_id": user_id,
            }
        )

    def rejoin(self, position, game_id) -> None:
        self.client.send_server(
            {
                "command": "rejoin",
                "p": position,
                "id": game_id,
            }
        )

    def leave(self, game_id) -> None:
        self.client.send_server(
            {
                "command": "leave",
                "id": game_id,
            }
        )

    def publish(self) -> None:
        self.client.send_server(
            {
                "command": "game_publish",
            }
        )

    def send_smile(self, smile_id: int = 16) -> None:
        self.client.send_server(
            {
                "command": "smile",
                "id": smile_id,
            }
        )

    def ready(self) -> None:
        self.client.send_server(
            {
                "command": "ready",
            }
        )

    def surrender(self) -> None:
        self.client.send_server(
            {
                "command": "surrender",
            }
        )

    def player_swap(self, position: int) -> None:
        self.send_server(
            {
                "command": "player_swap",
                "id": position,
            }
        )

    def turn(self, card: str) -> None:
        self.client.send_server(
            {
                "command": "t",
                "c": card,
            }
        )

    def take(self) -> None:
        self.client.send_server(
            {
                "command": "take",
            }
        )

    def _pass(self) -> None:
        self.client.send_server(
            {
                "command": "pass",
            }
        )


from setuptools import setup, find_packages

with open("README.md", "r") as stream:
    long_description = stream.read()

setup(
    name = "durakonline.py",
    version = "3.4.0",
    url = "https://github.com/Zakovskiy/durakonline.py",
    download_url = "https://github.com/Zakovskiy/durakonline.py/tarball/master",
    license = "MIT",
    author = "Zakovskiy",
    author_email = "gogrugu@gmail.com",
    description = "A library to create Durak Online bots.",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    keywords = [
        "durak",
        "online",
        "durakonline",
        "durakonline.py",
        "durakonline-bot",
        "rstgame",
        "rstgames",
        "api",
        "socket",
        "python",
        "python3",
        "python3.x",
        "zakovskiy",
        "official"
    ],
    install_requires = [
        "setuptools",
        "requests",
        "loguru",
        "aiohttp"
    ],
    setup_requires = [
        "wheel"
    ],
    packages = find_packages()
)
